//
// Module: EregionAnalyse
// Author: Richard Saxton 
//
// Description: Main class for calculating optimum image region
//
// Modifications: See ChangeLog
//

#ifndef EregionAnalyse_H
#define EregionAnalyse_H

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include <Tracer.h>
#include <metatask.h>
#include <errstr.h>
#include <param.h>
#include <Dal.h>
#include <Xmm.h>
#include "CalServer.h"

/* Local Classes */
#include "ImageRegion.h"
#include "CalSetter.h"


/****************************************************
 *
 * class EregionAnalyse
 *
 ***************************************************/

class EregionAnalyse
{
public:

  /* Constructor */
  EregionAnalyse();

  /* Destructor */
  ~EregionAnalyse();

  /* Define data structure */
  struct SOURCE {
      double bkgsub_counts,              // Source counts after bkgnd subt
             bkgsub_counts_error,        // Error on subtracted src counts
             bkgsub_count_rate,          // Bckgnd subtracted count rate
             bkgsub_count_rate_error,    // Error on subtracted count rate
             ee;                         // Applied encircled energy correction
  };

  /* Method to calculate the optimum radius */
  double EregionAnalyse::optimumRadius(double src_cnts, double inradius,
                     double bck_flux,
                     double energy, double offax,
                     double phi);

  /* Calculates the background subtracted source count rate and error */
  void EregionAnalyse::calcSrcCountRate(ImageRegion * srcreg,
                     ImageRegion * bckreg,
                     double bck_flux,
                     bool backgnd_region_set,
                     double energy, double exposure);

private:


  /* Member fields */
  SOURCE _source;
  ImageRegion * srcreg;
  ImageRegion * bckreg;
  ImageRegion * expreg;
  EncircledEnergyDataServer * _eds;
  EncircledEnergyFunction *eef;
  Psf *  _psf;
  CalSetter * _calset;

};

#endif

