// 
// Module: ImageRegion
// Author: Richard Saxton
//
// Description: 
//
// Given an input image, this class generates 
// a new PixelIterator for those pixels
// that are bounded by the specified region expression.
//
// Modifications: See ChangeLog
// 
// $Id: ImageRegion.h,v 1.5 2006/09/14 13:08:43 rsaxton Exp $

#ifndef ImageRegion_H
#define ImageRegion_H

// General classes
#include <iostream>

// SAS-specific classes
#include "PixelIterator.h"
#include "ImagePixelIterator.h"
#include "TablePixelIterator.h"
#include "PositionUtils.h"
#include "SourcePosition.h"
#include "DatafileAttributes.h"
#include "Xmm.h"

// General utility functions

// Local classes

class ImageRegion
{
public:

  /* Constructor */
  ImageRegion( const string & imagefile, const string & expression,
                        const string & tempfilename);

  /* Destructor */
  ~ImageRegion();

  /* Methods */
  void ImageRegion::init();
  void ImageRegion::interpretExpression(const string & expression);
  const void ImageRegion::setExtents();
  void ImageRegion::readkeywords();
  void ImageRegion::runAttCalc(const string & rawtablename);
  TablePixelIterator * ImageRegion::filterimage(  
                   ImagePixelIterator * fullimage, const string & expression );
  double ImageRegion::getRadius() const;
  double ImageRegion::exposure() const;
  double ImageRegion::counts() const;
  double ImageRegion::centralPixelValue();
  double ImageRegion::countsperpix();
  CoordPair ImageRegion::centroid(const int Niters );
  CoordPair ImageRegion::celestialCentre() const;
  const string ImageRegion::getCoordType(const string & expression);
  double ImageRegion::theta() const;
  double ImageRegion::phi() const;
  double ImageRegion::flux();
  double ImageRegion::number_of_pixels();
  double ImageRegion::pixelArea();
  double ImageRegion::area();
  double ImageRegion::xWidth();
  double ImageRegion::yWidth();
  double ImageRegion::asecToImgUnits();

  const WCSConvertor * const ImageRegion::setSkyWCS();

protected:

  const void ImageRegion::setUnits();
  const bool ImageRegion::checkaxes();

private:

  DataSet * _filteredset;
  DatafileAttributes * _imgAttribs;
  string _expr;                        // The image region spatial expression
  string _filteredtablename; 
  string _imagefile;
  bool _init;
  DataSet * set;
  DataSet * _imageset;
  TablePixelIterator *  _image;
  double _minX;
  double _maxX;
  double _maxY;
  double _minY;
  double _cenX;
  double _cenY;
  ImagePixelIterator * _fullimage;
  const PosWCS *      _poswcs;
  PositionUtils * _putils;
  const SourcePosition * _regCentre;
  string _expCoordType;
  string _imageCoordType;
  Array * _array;
  double _xUnits;
  double _yUnits;
  string               _instrumentID;
  string               _telescope;
  string               _datamode;
  string               _dateobs;
  string               _dateend;
  Xmm::Instrument       _instr;
  int                  _emdhlow;
  int                  _emdhupp;

  double _rapnt;
  double _decpnt;
  double _papnt;
  double _exposure;
};

#endif
